"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enum_builder_1 = require("./enum-builder");
const misc_1 = require("./misc");
class EnumMatcher {
    run(schema, column, viewName) {
        var _a;
        const enums = new enum_builder_1.EnumBuilder().run(schema);
        const candidates = enums.filter(e => e.table === viewName && column.field === e.field);
        if (candidates.length === 1) {
            return candidates[0].replacedBy || candidates[0];
        }
        const bestMatch = this.findCorresponding({
            field: column.field,
            table: viewName,
            options: column.enumValues || []
        }, enums);
        return ((_a = bestMatch) === null || _a === void 0 ? void 0 : _a.replacedBy) || bestMatch;
    }
    findCorresponding(view, tableEnums) {
        const matches = tableEnums.filter(t => misc_1.equalAsSets(t.options, view.options));
        if (matches.length === 1) {
            return matches[0];
        }
        matches.sort((a, b) => {
            const ascore = this.similarity(a.field, view.field);
            const bscore = this.similarity(b.field, view.field);
            return bscore - ascore;
        });
        return matches[0];
    }
    similarity(s1, s2) {
        var longer = s1;
        var shorter = s2;
        if (s1.length < s2.length) {
            longer = s2;
            shorter = s1;
        }
        var longerLength = longer.length;
        if (longerLength == 0) {
            return 1.0;
        }
        return (longerLength - this.editDistance(longer, shorter)) / parseFloat(longerLength + "");
    }
    editDistance(s1, s2) {
        s1 = s1.toLowerCase();
        s2 = s2.toLowerCase();
        var costs = new Array();
        for (var i = 0; i <= s1.length; i++) {
            var lastValue = i;
            for (var j = 0; j <= s2.length; j++) {
                if (i == 0)
                    costs[j] = j;
                else {
                    if (j > 0) {
                        var newValue = costs[j - 1];
                        if (s1.charAt(i - 1) != s2.charAt(j - 1))
                            newValue = Math.min(Math.min(newValue, lastValue), costs[j]) + 1;
                        costs[j - 1] = lastValue;
                        lastValue = newValue;
                    }
                }
            }
            if (i > 0)
                costs[s2.length] = lastValue;
        }
        return costs[s2.length];
    }
}
exports.EnumMatcher = EnumMatcher;
//# sourceMappingURL=enum-matcher.js.map