"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const changeCase = require("change-case");
const fs = require("fs");
const misc_1 = require("./misc");
const ql_enum_builder_1 = require("./ql-enum-builder");
class EnumWriter {
    run(enums, outputDir, qlOutputDir) {
        this.renderEnumString(enums, outputDir);
        ql_enum_builder_1.QlEnumBuilder.render(enums, qlOutputDir);
    }
    renderEnumString(enums, outputDir) {
        var _a;
        const dict = misc_1.groupBy(enums.filter(e => !e.replacedBy), "table");
        const grouped = Object.keys(dict).map(k => dict[k]);
        for (const arr of grouped) {
            const tableName = arr[0].table;
            const enumStrArr = arr.map(e => this.createEnum(e.field, e.options));
            if (enumStrArr.length) {
                let wrapper = this.wrapInNameSpace(tableName, enumStrArr);
                if (enumStrArr.length === 1 && ((_a = arr[0].replacementFor) === null || _a === void 0 ? void 0 : _a.length)) {
                    wrapper = enumStrArr[0];
                }
                const path = outputDir + "/" + changeCase.paramCase(tableName) + ".generated.ts";
                fs.writeFileSync(path, wrapper);
            }
        }
    }
    wrapInNameSpace(table, enumStr) {
        const name = changeCase.constantCase(table);
        return `export namespace ${name} {
  ${enumStr.join("\n\n")}
}`;
    }
    createEnum(column, values) {
        const rows = values.map(v => `${v} = "${v}"`);
        return `export enum ${changeCase.constantCase(column)} {
\t\t${rows.join(",\n\t\t")}
  }`;
    }
}
exports.EnumWriter = EnumWriter;
//# sourceMappingURL=enum-writer.js.map