"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const changeCase = require("change-case");
const fs = require("fs");
const misc_1 = require("./misc");
class QlEnumBuilder {
    static render(enums, outputDir) {
        const dict = misc_1.groupBy(enums.filter(e => !e.replacedBy), "table");
        const grouped = Object.keys(dict).map(k => dict[k]);
        for (const arr of grouped) {
            const tableName = arr[0].table;
            const intro = `import { GraphQLEnumType } from "graphql";`;
            if (!arr.length) {
                return;
            }
            const strExports = arr.map(e => this.createEnum(e.field, e.options)).join("\n\n");
            const output = `${intro}\n\n${strExports}\n`;
            const path = outputDir + "/" + changeCase.paramCase(tableName) + "-ql-enums.generated.ts";
            fs.writeFileSync(path, output);
        }
    }
    static createEnum(column, values) {
        const rows = values.map(v => {
            const qlCompliant = misc_1.valueToGraphqlCompliant(v);
            return `${qlCompliant}: { value: "${v}" }`;
        });
        return `const ${changeCase.camel(column)} = new GraphQLEnumType({
  name: "${changeCase.constantCase(column)}",
  values: {
    ${rows.join(",\n\t\t")}
  }
});
    
export { ${changeCase.camel(column)} as ${changeCase.pascal(column)}Enum };`;
    }
}
exports.QlEnumBuilder = QlEnumBuilder;
//# sourceMappingURL=ql-enum-builder.js.map